==> gspiceui-v0.8.90.tar.gz: gspiceui-v0.8.90/src/utility/SpinCtrl.cpp <==
//*****************************************************************************
//                                  SpinCtrl.cpp                              *
//                                 --------------                             *
//  Started     : 20/03/2004                                                  *
//  Last Update : 16/10/2006                                                  *
//  Copyright   : (C) 2004 by MSWaters                                        *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#include "utility/SpinCtrl.hpp"

//*****************************************************************************
// Implement an event table.

BEGIN_EVENT_TABLE( SpinCtrl, wxPanel )

  EVT_SPIN_UP  ( ID_SPINBTN, SpinCtrl::OnSbnScroll )
  EVT_SPIN_DOWN( ID_SPINBTN, SpinCtrl::OnSbnScroll )

END_EVENT_TABLE( )

//*****************************************************************************
// Constructor.
//
// Argument List:
//   eType - The type of the number to be displayed

SpinCtrl::SpinCtrl( eVarType eVType ) : wxPanel( )
{
  if( ! bSetVType( eVType ) ) // Set the         variable type to display
    m_eVType   = eVAR_FLT;    // Set the default variable type to display
  m_fInitValue =     0.0;     // Set the default initial value
  m_fMinValue  = -1000.0;     // Set the default minimum value
  m_fMaxValue  =  1000.0;     // Set the default maximum value
  m_fMinIncSz  =    10.0;     // Set the default minimum increment size
  m_fMaxIncSz  =    10.0;     // Set the default maximum increment size
}

//*****************************************************************************
// Destructor.

SpinCtrl::~SpinCtrl( )
{
}

//*****************************************************************************
// Layout the display objects.

void  SpinCtrl::DoLayout( void )
{
  wxBoxSizer * poSizer;
  wxSizerFlags  oFlags;

  poSizer = new wxBoxSizer( wxHORIZONTAL );

  oFlags.Expand( );
  oFlags.Proportion( 1 );
  poSizer->Add( &m_oTxtValue, oFlags );
  oFlags.Proportion( 0 );
  poSizer->Add( &m_oSbnValue, oFlags );

  SetSizer( poSizer );

  // Set minimum size and initial size as calculated by the sizer
  poSizer->SetSizeHints( this );
}

//*****************************************************************************
// Create an instance of this object.
//
// Argument List:
//   poWin  - The parent window
//   oWinID - The window identifier
//   iWidth - The width the spin control in pixels
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bCreate( wxWindow * poWin, wxWindowID oWinID, int iWidth )
{
  wxSize  oSize=wxDefaultSize;

  if( bIsCreated( ) )             return( TRUE );

  // Create the display objects
  if( ! Create( poWin, oWinID ) ) return( FALSE );
  if( iWidth > 0 ) oSize.SetWidth( iWidth );
  m_oTxtValue.Create( this, ID_TEXTCTL, wxT(""), wxDefaultPosition,
                      oSize,         wxTE_LEFT );
  m_oSbnValue.Create( this, ID_SPINBTN,          wxDefaultPosition,
                      wxDefaultSize, wxSP_VERTICAL | wxSP_ARROW_KEYS );

  // Set the display object attributes
  m_oSbnValue.SetRange( -0x8000, 0x7FFF );
  bSetParms( m_fInitValue, m_fMinValue, m_fMaxValue, m_fMinIncSz, m_fMaxIncSz );
  bSetValue( m_fInitValue );

  // Layout the display objects
  DoLayout( );

  return( TRUE );
}

//*****************************************************************************
// Clear the object attributes.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bClear( void )
{
  return( bSetValue( m_fInitValue ) );
}

//*****************************************************************************
// Set the variable type to displayed by the spin control.
//
// Argument List:
//   eVType - The variable type specifier
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bSetVType( eVarType eVType )
{
  float  f1;

  // Argument validity checks
  if( m_eVType == eVType )                 return( TRUE );
  if( eVType<eVAR_FST || eVType>eVAR_LST ) return( FALSE );

  m_eVType = eVType; // Set the new variable type

  // Display the new variable type
  if( bIsCreated( ) )
  {
    f1 = fGetValue( );
    if( ! bSetValue( f1 ) )                return( FALSE );
  }

  return( TRUE );
}

//*****************************************************************************
// Set the current value of the spin control.
//
// Argument List:
//   fValue - The float value
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bSetValue( float fValue )
{
  wxString  osValue, os1;
  float     f1;
  int       i1;

  if( ! bIsCreated( ) )           return( FALSE );
  if( (m_eVType==eVAR_SCI || m_eVType==eVAR_ENG) && fValue==0.0 ) ;
  else if( fValue < m_fMinValue ) return( FALSE );
  if( fValue > m_fMaxValue )      return( FALSE );

  m_oTxtValue.Clear( );
  switch( m_eVType )
  {
    case eVAR_HEX:
      if( fValue < 0.0 )                 return( FALSE );
      i1 = (int) ( fValue + 0.5 );
      osValue.Printf( wxT("%X"), i1 );
      break;

    case eVAR_INT:
      if( fValue >= 0.0 ) i1 = (int) ( fValue + 0.5 );
      else                i1 = (int) ( fValue - 0.5 );
      osValue << i1;
      break;

    case eVAR_FLT:
      if( fabs( fValue ) < 1000.0 ) os1 = wxT("%#.2f");
      else                          os1 = wxT("%#.1f");
      osValue.Printf( os1.c_str( ), fValue );
      break;

    case eVAR_SCI:
      osValue.Printf( wxT("%#.2E"), fValue );
      break;

    case eVAR_ENG:
      if( ! ConvertType::bParseFlt( (double) fValue, &f1, &i1 ) )
        return( FALSE );
      while( i1 % 3 ) { f1 *= 10.0; i1--; }
      osValue.Printf( wxT("%#.2fE%02i"), f1, i1 );
      break;

    default:                             return( FALSE );
  }

  osValue.Prepend( wxT(" ") );
  m_oTxtValue.SetValue( osValue );

  return( TRUE );
}

//*****************************************************************************
// Set the current value of the spin control.
//
// Argument List:
//   fValue - The string value
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bSetValue( const wxString & rosValue )
{
  double  fValue;

  if( ! ConvertType::bStrToFlt( rosValue, &fValue ) ) return( FALSE );
  if( ! bSetValue( (float) fValue ) )                 return( FALSE );

  return( TRUE );
}
//*****************************************************************************
// Set the initial value of the spin control.
//
// Argument List:
//   fInitValue - The spin control initial value
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bSetInitValue( float fInitValue )
{
  if( fInitValue<m_fMinValue || fInitValue>m_fMaxValue ) return( FALSE );

  m_fInitValue = fInitValue;

  return( TRUE );
}

//*****************************************************************************
// Set the increment sizes of the spin control.
//
// This spin control can be incremented using two different approaches,
// constant or variable step sizes. A constant step size means that the spin
// control is incremented by the same amount throughout it's range. A variable
// step size means that the spin control is incremented by an amount dependent
// on it's current value.
//
// Argument List:
//   fMinIncSz - The minimum spin control increment size
//   fMaxIncSz - The maximum spin control increment size
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bSetIncSz( float fMinIncSz, float fMaxIncSz )
{
   // Constant or variable incrementing?
  if( fMaxIncSz <= 0.0 ) fMaxIncSz = fMinIncSz;

  // Do some validity checks on the arguments
  if( fMinIncSz > fMaxIncSz )                   return( FALSE );
  // Check that the new increment sizes fit within the current range
  if( fMaxIncSz > (m_fMaxValue - m_fMinValue) ) return( FALSE );

  m_fMinIncSz = fMinIncSz; // Set increment sizes
  m_fMaxIncSz = fMaxIncSz;

  return( TRUE );
}

//*****************************************************************************
// Set the spincontrol range.
//
// Argument List:
//   fMinValue - The spin control minimum value
//   fMaxValue - The spin control maximum value
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bSetRange( float fMinValue, float fMaxValue )
{
  return( bSetParms( m_fInitValue, fMinValue, fMaxValue ) );
}

//*****************************************************************************
// Set all the spin control parameters.
//
// Argument List:
//   fInitValue - The spin control initial value
//   fMinValue  - The spin control minimum value
//   fMaxValue  - The spin control maximum value
//   fMinIncSz  - The spin control minimum increment size
//   fMaxIncSz  - The spin control maximum increment size
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bSetParms( float fInitValue, float fMinValue, float fMaxValue,
                           float fMinIncSz, float fMaxIncSz )
{
  wxString  os1;
  double    f1, f2;

  if( fMinIncSz > 0.0 ) f1 = fMinIncSz;
  else                  f1 = fMinValue / 10.0;
  if( fMaxIncSz > 0.0 ) f2 = fMaxIncSz;
  else                  f2 = fMaxValue / 10.0;

  // Check argument validity
  if( fMinValue > fMaxValue )                        return( FALSE );
  if( fMinIncSz > fMaxIncSz )                        return( FALSE );
  if( f2 > (fMaxValue-fMinValue) )                   return( FALSE );
  if( m_eVType!=eVAR_SCI && m_eVType!=eVAR_ENG && fInitValue!=0.0 )
    if( fInitValue<fMinValue || fInitValue>fMaxValue )
      return( FALSE );

  // Set new parameter values
  m_fInitValue = fInitValue;
  m_fMinValue  = fMinValue;
  m_fMaxValue  = fMaxValue;

  // Update the text control if necessary
  if( bIsCreated( ) )
  {
    os1 = m_oTxtValue.GetValue( );
    ConvertType::bStrToFlt( os1, &f1 );
         if( f1 < m_fMinValue ) bSetValue( m_fMinValue );
    else if( f1 > m_fMaxValue ) bSetValue( m_fMaxValue );
  }

  if( ! bSetIncSz( f1, f2 ) )        return( FALSE );

  return( TRUE );
}

//*****************************************************************************
// Get the current spin control value as an integer.
//
// Return Values:
//   Success - The integer value
//   Failure - 0

int  SpinCtrl::iGetValue( void )
{
  float  f1;
  int    i1;

  if( ! bIsCreated( ) ) return( 0 );

  f1 = fGetValue( );
  if( f1 >= 0.0 ) i1 = (int) ( f1 + 0.5 );
  else            i1 = (int) ( f1 - 0.5 );

  return( i1 );
}
//*****************************************************************************
// Get the current spin control value as a float.
//
// Return Values:
//   Success - The float value
//   Failure - 0.0

float  SpinCtrl::fGetValue( void )
{
  wxString  os1;
  double    f1;

  if( ! bIsCreated( ) ) return( 0.0 );

  os1 = m_oTxtValue.GetValue( );
  f1 = 0.0;
  ConvertType::bStrToFlt( os1 , &f1 );

  return( f1 );
}

//*****************************************************************************
// Get the current spin control value as a string.
//
// Return Values:
//   Success - The string value
//   Failure - An empty string

const wxString & SpinCtrl::rosGetValue( void )
{
  static  wxString  osValue;
  double  f1;

  osValue.Empty( );

  if( ! bIsCreated( ) ) return( osValue );

  osValue = m_oTxtValue.GetValue( );
  if( ! osValue.ToDouble( &f1 ) ) osValue.Empty( );

  return( osValue );
}

//*****************************************************************************
//
//                             Event Handlers
//
//*****************************************************************************
// Spin button scroll event handler.
//
// Argument List:
//   roEvtSpn - An object holding information about the event

void  SpinCtrl::OnSbnScroll( wxSpinEvent & roEvtSpn )
{
  wxString  os1;
  double    df1;
  float     fValue, fIncSz, fEdit, f1;
  int       i1;

  // Get the current text control value
  os1 = m_oTxtValue.GetValue( );
  df1 = 0.0;
  if( ! ConvertType::bStrToFlt( os1, &df1 ) )     return;
  if( ! ConvertType::bParseFlt( df1, &f1, &i1 ) ) return;
  fValue = (float) df1;

  // Determine the increment size to use
  if( roEvtSpn.GetEventType( )==wxEVT_SCROLL_LINEDOWN && f1== 1.0 ) i1--;
  if( roEvtSpn.GetEventType( )==wxEVT_SCROLL_LINEUP   && f1==-1.0 ) i1--;  
  if( m_eVType==eVAR_SCI || m_eVType==eVAR_ENG )
  {
    fIncSz = (float) pow( 10.0, (double) i1-1 );
    if( fValue == 0.0 ) fIncSz = m_fMinValue;
  }
  else
  {
    fIncSz = (float) pow( 10.0, (double) i1 );
    if( fIncSz < m_fMinIncSz ) fIncSz = m_fMinIncSz;
    if( fIncSz > m_fMaxIncSz ) fIncSz = m_fMaxIncSz;
  }

  // Determine if the text control has been altered by the user
  fEdit = fValue - ( (int) (fValue / fIncSz) ) * fIncSz;
  if( fabs( fEdit ) < fabs( fEdit - fIncSz ) ) f1 = fabs( fEdit );
  else                                         f1 = fabs( fEdit - fIncSz );
  if( m_eVType==eVAR_SCI || m_eVType==eVAR_ENG ) f1 /= (10.0 * fIncSz);
  if( f1 < 0.01 ) fEdit = 0.0;

  // Determine the event type and act accordingly
  if( roEvtSpn.GetEventType( ) == wxEVT_SCROLL_LINEUP )
  { // Increment the value
    if( fValue < m_fMaxValue )
    {
      fValue += fIncSz;
      if( fEdit != 0.0 ) // The value has been edited by the user
        { fEdit > 0.0 ? fValue -= fEdit : fValue -= fEdit + fIncSz; }
      if( fValue > m_fMaxValue ) fValue = m_fMaxValue;
    }
    else if( m_eVType==eVAR_SCI || m_eVType==eVAR_ENG )
      if( fValue < m_fMinValue ) fValue = m_fMinValue;
  }
  else if( roEvtSpn.GetEventType( ) == wxEVT_SCROLL_LINEDOWN )
  { // Decrement the value
    if( fValue > m_fMinValue )
    {
      fValue -= fIncSz;
      if( fEdit != 0.0 ) // The value has been edited by the user
        { fEdit > 0.0 ? fValue -= fEdit - fIncSz : fValue -= fEdit; }
      if( fValue < m_fMinValue ) fValue = m_fMinValue;
    }
    else if( m_eVType==eVAR_SCI || m_eVType==eVAR_ENG ) fValue = 0.0;
  }
  else return;

  bSetValue( fValue ); // Set the text control value

  roEvtSpn.Skip( ); // Continue to search for event handlers
}

//*****************************************************************************

==> gspiceui-v0.9.33.tar.gz: gspiceui-v0.9.33/src/utility/SpinCtrl.cpp <==
//*****************************************************************************
//                                  SpinCtrl.cpp                              *
//                                 --------------                             *
//  Started     : 20/03/2004                                                  *
//  Last Update : 26/06/2007                                                  *
//  Copyright   : (C) 2004 by MSWaters                                        *
//  Email       : M.Waters@bom.gov.au                                         *
//*****************************************************************************

//*****************************************************************************
//                                                                            *
//    This program is free software; you can redistribute it and/or modify    *
//    it under the terms of the GNU General Public License as published by    *
//    the Free Software Foundation; either version 2 of the License, or       *
//    (at your option) any later version.                                     *
//                                                                            *
//*****************************************************************************

#include "utility/SpinCtrl.hpp"

//*****************************************************************************
// Implement an event table.

BEGIN_EVENT_TABLE( SpinCtrl, wxPanel )

  EVT_SPIN_UP  ( ID_SPINBTN, SpinCtrl::OnSbnScroll )
  EVT_SPIN_DOWN( ID_SPINBTN, SpinCtrl::OnSbnScroll )

END_EVENT_TABLE( )

//*****************************************************************************
// Constructor.
//
// Argument List:
//   eVType - The type of the variable to be displayed

SpinCtrl::SpinCtrl( eVarType eVType ) : wxPanel( )
{
  if( ! bSetVarType( eVType ) ) // Set the         variable type to display
    m_eVarType = eVAR_FLT;      // Set the default variable type to display
  m_fDefValue  =     0.0;       // Set the default value
  m_fMinValue  = -1000.0;       // Set the default minimum value
  m_fMaxValue  =  1000.0;       // Set the default maximum value
  m_fMinIncSz  =    10.0;       // Set the default minimum increment size
  m_fMaxIncSz  =    10.0;       // Set the default maximum increment size
}

//*****************************************************************************
// Destructor.

SpinCtrl::~SpinCtrl( )
{
}

//*****************************************************************************
// Layout the display objects.

void  SpinCtrl::DoLayout( void )
{
  wxBoxSizer * poSizer;
  wxSizerFlags  oFlags;

  poSizer = new wxBoxSizer( wxHORIZONTAL );

  oFlags.Expand( );
  oFlags.Proportion( 1 );
  poSizer->Add( &m_oTxtValue, oFlags );
  oFlags.Proportion( 0 );
  poSizer->Add( &m_oSbnValue, oFlags );

  SetSizer( poSizer );

  // Set minimum size and initial size as calculated by the sizer
  poSizer->SetSizeHints( this );
}

//*****************************************************************************
// Create an instance of this object.
//
// Argument List:
//   poWin  - The parent window
//   oWinID - The window identifier
//   iWidth - The width the spin control in pixels
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bCreate( wxWindow * poWin, wxWindowID oWinID, int iWidth )
{
  wxSize  oSize=wxDefaultSize;

  if( bIsCreated( ) )             return( TRUE );

  // Create the display objects
  if( ! Create( poWin, oWinID ) ) return( FALSE );
  if( iWidth > 0 ) oSize.SetWidth( iWidth );
  m_oTxtValue.Create( this, ID_TEXTCTL, wxT(""), wxDefaultPosition,
                      oSize,         wxTE_RIGHT );
  m_oSbnValue.Create( this, ID_SPINBTN,          wxDefaultPosition,
                      wxDefaultSize, wxSP_VERTICAL | wxSP_ARROW_KEYS );

  // Set the display object attributes
  m_oSbnValue.SetRange( -0x8000, 0x7FFF );
  bSetParms( m_fDefValue, m_fMinValue, m_fMaxValue, m_fMinIncSz, m_fMaxIncSz );
  bSetValue( m_fDefValue );

  // Layout the display objects
  DoLayout( );

  return( TRUE );
}

//*****************************************************************************
// Clear the object attributes.
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bClear( void )
{
  return( bSetValue( m_fDefValue ) );
}

//*****************************************************************************
// Set the variable type to displayed by the spin control.
//
// Argument List:
//   eVType - The variable type specifier
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bSetVarType( eVarType eVType )
{
  float  f1;

  // Argument validity checks
  if( m_eVarType == eVType )               return( TRUE );
  if( eVType<eVAR_FST || eVType>eVAR_LST ) return( FALSE );

  m_eVarType = eVType; // Set the new variable type

  // Display the new variable type
  if( bIsCreated( ) )
  {
    f1 = fGetValue( );
    if( ! bSetValue( f1 ) )                return( FALSE );
  }

  return( TRUE );
}

//*****************************************************************************
// Set the current value of the spin control.
//
// Argument List:
//   fValue - The float value
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bSetValue( float fValue )
{
  wxString  osValue, os1;
  float     f1;
  int       i1;

  if( ! bIsCreated( ) )           return( FALSE );
  if( (m_eVarType==eVAR_SCI || m_eVarType==eVAR_ENG) && fValue==0.0 ) ;
  else if( fValue < m_fMinValue ) return( FALSE );
  if( fValue > m_fMaxValue )      return( FALSE );

  m_oTxtValue.Clear( );
  switch( m_eVarType )
  {
    case eVAR_HEX:
      if( fValue < 0.0 )                 return( FALSE );
      i1 = (int) ( fValue + 0.5 );
      osValue.Printf( wxT("%X"), i1 );
      break;

    case eVAR_INT:
      if( fValue >= 0.0 ) i1 = (int) ( fValue + 0.5 );
      else                i1 = (int) ( fValue - 0.5 );
      osValue << i1;
      break;

    case eVAR_FLT:
      if( fabs( fValue ) < 1000.0 ) os1 = wxT("%#.2f");
      else                          os1 = wxT("%#.1f");
      osValue.Printf( os1.c_str( ), fValue );
      break;

    case eVAR_SCI:
      osValue.Printf( wxT("%#.2E"), fValue );
      break;

    case eVAR_ENG:
      if( ! ConvertType::bParseFlt( (double) fValue, &f1, &i1 ) )
        return( FALSE );
      while( i1 % 3 ) { f1 *= 10.0; i1--; }
      osValue.Printf( wxT("%#.2fE%02i"), f1, i1 );
      break;

    default:                             return( FALSE );
  }

// osValue.Prepend( wxT(" ") ); ???
  osValue. Append( wxT(" ") );
  m_oTxtValue.SetValue( osValue );

  return( TRUE );
}

//*****************************************************************************
// Set the current value of the spin control.
//
// Argument List:
//   fValue - The string value
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bSetValue( const wxString & rosValue )
{
  double  fValue;

  if( ! ConvertType::bStrToFlt( rosValue, &fValue ) ) return( FALSE );
  if( ! bSetValue( (float) fValue ) )                 return( FALSE );

  return( TRUE );
}
//*****************************************************************************
// Set the initial value of the spin control.
//
// Argument List:
//   fDefValue - The spin control default value
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bSetDefValue( float fDefValue )
{
  if( fDefValue<m_fMinValue || fDefValue>m_fMaxValue ) return( FALSE );

  m_fDefValue = fDefValue;

  return( TRUE );
}

//*****************************************************************************
// Set the increment sizes of the spin control.
//
// This spin control can be incremented using two different approaches,
// constant or variable step sizes. A constant step size means that the spin
// control is incremented by the same amount throughout it's range. A variable
// step size means that the spin control is incremented by an amount dependent
// on it's current value.
//
// Argument List:
//   fMinIncSz - The minimum spin control increment size
//   fMaxIncSz - The maximum spin control increment size
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bSetIncSz( float fMinIncSz, float fMaxIncSz )
{
   // Constant or variable incrementing?
  if( fMaxIncSz <= 0.0 ) fMaxIncSz = fMinIncSz;

  // Do some validity checks on the arguments
  if( fMinIncSz > fMaxIncSz )                   return( FALSE );
  // Check that the new increment sizes fit within the current range
  if( fMaxIncSz > (m_fMaxValue - m_fMinValue) ) return( FALSE );

  m_fMinIncSz = fMinIncSz; // Set increment sizes
  m_fMaxIncSz = fMaxIncSz;

  return( TRUE );
}

//*****************************************************************************
// Set the spincontrol range.
//
// Argument List:
//   fMinValue - The spin control minimum value
//   fMaxValue - The spin control maximum value
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bSetRange( float fMinValue, float fMaxValue )
{
  return( bSetParms( m_fDefValue, fMinValue, fMaxValue ) );
}

//*****************************************************************************
// Set all the spin control parameters.
//
// Argument List:
//   fDefValue - The spin control default value
//   fMinValue - The spin control minimum value
//   fMaxValue - The spin control maximum value
//   fMinIncSz - The spin control minimum increment size (must be positive)
//   fMaxIncSz - The spin control maximum increment size (must be positive)
//
// Return Values:
//   TRUE  - Success
//   FALSE - Failure

bool  SpinCtrl::bSetParms( float fDefValue, float fMinValue, float fMaxValue,
                           float fMinIncSz, float fMaxIncSz )
{
  wxString  os1;
  double    df1;

  // Check argument validity
  if( fMinValue >= fMaxValue )                       return( FALSE );
  if( m_eVarType!=eVAR_SCI && m_eVarType!=eVAR_ENG && fDefValue!=0.0 )
    if( fDefValue<fMinValue || fDefValue>fMaxValue ) return( FALSE );
  if( fMinIncSz > fMaxIncSz )                        return( FALSE );
  if( fMaxIncSz > (fMaxValue - fMinValue) )          return( FALSE );
  if( fMinIncSz<=0.0 || fMaxIncSz<=0.0 )             return( FALSE );

  // Set new parameter values
  m_fDefValue = fDefValue;
  m_fMinValue = fMinValue;
  m_fMaxValue = fMaxValue;
  if( ! bSetIncSz( fMinIncSz, fMaxIncSz ) )          return( FALSE );

  // Update the text control if necessary
  if( bIsCreated( ) )
  {
    os1 = m_oTxtValue.GetValue( );
    ConvertType::bStrToFlt( os1, &df1 );
         if( df1 < m_fMinValue ) bSetValue( m_fMinValue );
    else if( df1 > m_fMaxValue ) bSetValue( m_fMaxValue );
  }

  return( TRUE );
}

//*****************************************************************************
// Get the current spin control value as an integer.
//
// Return Values:
//   Success - The integer value
//   Failure - 0

int  SpinCtrl::iGetValue( void )
{
  float  f1;
  int    i1;

  if( ! bIsCreated( ) ) return( 0 );

  f1 = fGetValue( );
  if( f1 >= 0.0 ) i1 = (int) ( f1 + 0.5 );
  else            i1 = (int) ( f1 - 0.5 );

  return( i1 );
}
//*****************************************************************************
// Get the current spin control value as a float.
//
// Return Values:
//   Success - The float value
//   Failure - 0.0

float  SpinCtrl::fGetValue( void )
{
  wxString  os1;
  double    f1;

  if( ! bIsCreated( ) ) return( 0.0 );

  os1 = m_oTxtValue.GetValue( );
  f1 = 0.0;
  ConvertType::bStrToFlt( os1 , &f1 );

  return( f1 );
}

//*****************************************************************************
// Get the current spin control value as a string.
//
// Return Values:
//   Success - The string value
//   Failure - An empty string

const wxString & SpinCtrl::rosGetValue( void )
{
  static  wxString  osValue;
  double  df1;

  osValue.Empty( );

  if( ! bIsCreated( ) ) return( osValue );

  osValue = m_oTxtValue.GetValue( );
  osValue.Trim( );
  if( ! osValue.ToDouble( &df1 ) ) osValue.Empty( );

  return( osValue );
}

//*****************************************************************************
//
//                             Event Handlers
//
//*****************************************************************************
// Spin button scroll event handler.
//
// Argument List:
//   roEvtSpn - An object holding information about the event

void  SpinCtrl::OnSbnScroll( wxSpinEvent & roEvtSpn )
{
  wxString  os1;
  double    df1;
  float     fValue, fIncSz, fEdit, f1;
  int       i1;

  // Get the current text control value
  os1 = m_oTxtValue.GetValue( );
  df1 = 0.0;
  if( ! ConvertType::bStrToFlt( os1, &df1 ) )     return;
  if( ! ConvertType::bParseFlt( df1, &f1, &i1 ) ) return;
  fValue = (float) df1;

  // Determine the increment size to use
  if( roEvtSpn.GetEventType( )==wxEVT_SCROLL_LINEDOWN && f1== 1.0 ) i1--;
  if( roEvtSpn.GetEventType( )==wxEVT_SCROLL_LINEUP   && f1==-1.0 ) i1--;  
  if( m_eVarType==eVAR_SCI || m_eVarType==eVAR_ENG )
  {
    fIncSz = (float) pow( 10.0, (double) i1-1 );
    if( fValue == 0.0 ) fIncSz = m_fMinValue;
  }
  else
  {
    fIncSz = (float) pow( 10.0, (double) i1 );
    if( fIncSz < m_fMinIncSz ) fIncSz = m_fMinIncSz;
    if( fIncSz > m_fMaxIncSz ) fIncSz = m_fMaxIncSz;
  }

  // Determine if the text control has been altered by the user
  fEdit = fValue - ( (int) (fValue / fIncSz) ) * fIncSz;
  if( fabs( fEdit ) < fabs( fEdit - fIncSz ) ) f1 = fabs( fEdit );
  else                                         f1 = fabs( fEdit - fIncSz );
  if( m_eVarType==eVAR_SCI || m_eVarType==eVAR_ENG ) f1 /= (10.0 * fIncSz);
  if( f1 < 0.01 ) fEdit = 0.0;

  // Determine the event type and act accordingly
  if( roEvtSpn.GetEventType( ) == wxEVT_SCROLL_LINEUP )
  { // Increment the value
    if( fValue < m_fMaxValue )
    {
      fValue += fIncSz;
      if( fEdit != 0.0 ) // The value has been edited by the user
        { fEdit > 0.0 ? fValue -= fEdit : fValue -= fEdit + fIncSz; }
      if( fValue > m_fMaxValue ) fValue = m_fMaxValue;
    }
    else if( m_eVarType==eVAR_SCI || m_eVarType==eVAR_ENG )
      if( fValue < m_fMinValue ) fValue = m_fMinValue;
  }
  else if( roEvtSpn.GetEventType( ) == wxEVT_SCROLL_LINEDOWN )
  { // Decrement the value
    if( fValue > m_fMinValue )
    {
      fValue -= fIncSz;
      if( fEdit != 0.0 ) // The value has been edited by the user
        { fEdit > 0.0 ? fValue -= fEdit - fIncSz : fValue -= fEdit; }
      if( fValue < m_fMinValue ) fValue = m_fMinValue;
    }
    else if( m_eVarType==eVAR_SCI || m_eVarType==eVAR_ENG ) fValue = 0.0;
  }
  else return;

  bSetValue( fValue ); // Set the text control value

  roEvtSpn.Skip( ); // Continue to search for event handlers
}

//*****************************************************************************
